function [varargout] = GenWaitbar(n,varargin)
%GENWAITBAR Erzeugt eine einfache Statusanzeige mit Abbruchsbutton
% 
%  - Genwaitbar(x) erzeugt eine Waitbar mit dem Text 
%    'Generation wird berechnet', deren Laenge mit x angegeben wird. 
%
%  - x muss zwischen 0 und 1 liegen.
%
%  - mit x = 0 wird die Waitbar initialisiert
%
%  - Genwaitbar(0,n) initialisiert eine Waitbar, deren Text 'n. Generation 
%    wird berechnet...' lautet. 
%     
%  - mit x = 1 wird die Waitbar geschlossen, falls dies noch nicht manuell
%    erfolgt ist.
%
%  - b = Genwaitbar(x): Wird der Abbruch-Button auf der Waitbar bettigt,
%    wechselt der optional nutzbare Ausgang b von logisch 'false' auf 'true'.
% 
%  - Wird die Berechnung ber den Button abgebrochen, so kann es einen Moment 
%    (je nach Hufigkeit der Abfrage des Status b) bis die Berechnung
%    tatschlich abbricht. Dies wird ebenfalls mit einer Waitbar angezeigt.
%    Wird diese Waitbar geschlossen, so erfolgt der Abbruch doch nicht!

persistent HANDLE_WAITBAR

if n==0 % Initialisiere waitbar
    if nargin ==1
        HANDLE_WAITBAR = waitbar(n,'Generation wird berechnet...             ',...
        'CreateCancelBtn',{@CB_CancelButton,HANDLE_WAITBAR},...
        'CloseRequestFcn',@my_closefcn);
    else
        str = [num2str(varargin{1}),'. Generation wird berechnet...             '];
        HANDLE_WAITBAR = waitbar(n,str,...
        'CreateCancelBtn',{@CB_CancelButton,HANDLE_WAITBAR},...
        'CloseRequestFcn',@my_closefcn);
    end
        Abbruch = false;        % Bisher kein Abbruch erfolgt
        guidata(HANDLE_WAITBAR,Abbruch); % speichern
    return
end
if (0<n)&&(n<=1)
    if ~isempty(HANDLE_WAITBAR)% Figure wurde noch nicht geschlossen
        Fenster_offen=1;
        try % ist das Fenster geschlossen worden ohne die Generation abzubrechen?
            Abbruch = guidata(HANDLE_WAITBAR); 
        catch % ...dann
            HANDLE_WAITBAR=[];
            Fenster_offen=0;
            Abbruch = false;
        end
        if ((Abbruch == false) && Fenster_offen)
            waitbar(n,HANDLE_WAITBAR)
        end
        if (n==1)||(Abbruch == true)
            close(HANDLE_WAITBAR);
            HANDLE_WAITBAR=[];
        end
    else
        Abbruch = 0; % Eigentlich unbekannt
    end
    if nargout == 1
        varargout(1) = {Abbruch};
    end
    return
else
    error('GenWaitbar untersttzt nur ein skalares Eingangsargument zwischen 0 und 1')%
end

end

function CB_CancelButton(src,evnt,h)
selection = questdlg('Berechnung dieser Generation abbrechen?          .',...
    '','Ja','Nein','Ja');
switch selection,
    case 'Ja',
        Abbruch = true;
        guidata(src,Abbruch);
        h = gcf;
        waitbar(1,h,'Berechnung der Generation wird abgebrochen...                .')
        % Wenn diese Abbruchs-Waitbar geschlossen wird, erfolgt der Abbruch nicht!
    case 'Nein'
        return
end
end

function my_closefcn(src,evnt)
delete(src)
end
